﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;

using System.Web;
using System.Xml.Serialization;
using System.Xml;

namespace Com.Piwigo.Lib.Proxy
{

    static public class PwgGenericProxy<T>
    {
        
        public static T Get(Uri address, String methodName, String methodParameter)
        {
            T ReturnValue = default(T);
            HttpWebRequest Request;
            HttpWebResponse Response = null;
            StreamReader Reader;

            if (address == null) { throw new PwgProxyException("The get address is null."); }

            try
            {
                UriBuilder completeUri = new UriBuilder(address);
                if (methodParameter == null)
                {
                    completeUri.Query = String.Format("method={0}", methodName);
                }
                else
                {
                    completeUri.Query = String.Format("method={0}&{1}", methodName, methodParameter);
                }
                // Create and initialize the web request  
                Request = WebRequest.Create(completeUri.Uri) as HttpWebRequest;
                Request.UserAgent = PwgConfigProxy.PwgUserAgent;
                Request.KeepAlive = PwgConfigProxy.PwgKeepAlive;
                Request.Timeout = PwgConfigProxy.PwgTimeOutInSecond * 1000;
                Request.CookieContainer = PwgConfigProxy.cookieContainer; // get session cookie

                // Get response  
                Response = Request.GetResponse() as HttpWebResponse;

                if (Request.HaveResponse == true && Response != null)
                {
                    // Get the response stream  
                    Reader = new StreamReader(Response.GetResponseStream());

                    ReturnValue = DeserializeObject(Reader);
                }
            }
            catch (WebException wex)
            {
                // This exception will be raised if the server didn't return 200 - OK  
                // Try to retrieve more information about the network error  
                if (wex.Response != null)
                {
                    using (HttpWebResponse errorResponse = (HttpWebResponse)wex.Response)
                    {
                        throw new PwgProxyException(
                            String.Format(
                                "The server returned '{0}' with the status code {1} ({2:d}).",
                                errorResponse.StatusDescription, errorResponse.StatusCode,
                                errorResponse.StatusCode),
                                wex);
                    }
                }
            }
            finally
            {
                if (Response != null) { Response.Close(); }
            }
            return ReturnValue;
        }

        public static T Post(Uri address, String data)
        {
            T ReturnValue = default(T);

            HttpWebRequest Request;
            HttpWebResponse Response = null;

            if (address == null) { throw new PwgProxyException("The get address is null."); }

            try
            {
                // Create the web request  
                Request = WebRequest.Create(address) as HttpWebRequest;
                Request.UserAgent = PwgConfigProxy.PwgUserAgent;
                Request.KeepAlive = PwgConfigProxy.PwgKeepAlive;
                Request.Timeout = PwgConfigProxy.PwgTimeOutInSecond * 1000;
                Request.CookieContainer = PwgConfigProxy.cookieContainer; // get session cookie

                // Set type to POST  
                Request.Method = "POST";
                Request.ContentType = "application/x-www-form-urlencoded";

                // Create a byte array of the data we want to send  
                byte[] byteData = UTF8Encoding.UTF8.GetBytes(data);

                // Set the content length in the request headers  
                Request.ContentLength = byteData.Length;

                // Write data  
                using (Stream PostStream = Request.GetRequestStream())
                {
                    PostStream.Write(byteData, 0, byteData.Length);
                }

                // Get response  
                using (Response = Request.GetResponse() as HttpWebResponse)
                {
                    // Get the response stream  
                    StreamReader reader = new StreamReader(Response.GetResponseStream());

                    ReturnValue = DeserializeObject(reader);
                }
            }
            catch (WebException wex)
            {
                // This exception will be raised if the server didn't return 200 - OK  
                // Try to retrieve more information about the network error  
                if (wex.Response != null)
                {
                    using (HttpWebResponse ErrorResponse = (HttpWebResponse)wex.Response)
                    {
                        throw new PwgProxyException(
                            String.Format(
                                "The server returned '{0}' with the status code {1} ({2:d}).",
                                ErrorResponse.StatusDescription, ErrorResponse.StatusCode,
                                ErrorResponse.StatusCode),
                            wex);
                    }
                }
            }
            finally
            {
                if (Response != null) { Response.Close(); }
            }
            return ReturnValue;
        }

        public static T DeserializeObject(StreamReader xmlData)
        {
            T ReturnValue = default(T);

            try
            {
                //Console.WriteLine(xmlData.ReadToEnd());

                System.Xml.Serialization.XmlSerializer xsSerializer = new System.Xml.Serialization.XmlSerializer(typeof(T));
                ReturnValue = (T)xsSerializer.Deserialize(xmlData);
            }
            catch (Exception ex)
            {
                throw new PwgProxyException(
                            String.Format(
                                "The value {0} can't be deserialized in {1}, the error is {2} .",
                                xmlData, typeof(T).Name, ex.Message),
                            ex);
            }

            return ReturnValue;
        }

        public static void SerializeToXML(String fileName,T obj  )
        {
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(T));
                TextWriter textWriter = new StreamWriter(fileName);
                serializer.Serialize(textWriter, obj);
                textWriter.Close();
            }
            catch (Exception ex)
            {
                throw new PwgProxyException(
                            String.Format(
                                "The value {0} can't be serialized in {1}, the error is {2} .",
                                obj, fileName, ex.Message),
                            ex);
            }
        }
    }
}
